@@ -9,10 +9,15 @@ import attr
 import pkg_resources
 import structlog
 
-from PyQt5.QtCore import QUrl, QTimer
+from PyQt5.QtCore import QUrl, QTimer, pyqtSlot, Qt
 from PyQt5.QtNetwork import QNetworkCookie, QNetworkProxy
-from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineScript, QWebEngineProfile
-from PyQt5.QtWidgets import QApplication
+from PyQt5.QtWebEngineWidgets import (
+    QWebEngineView,
+    QWebEngineScript,
+    QWebEngineProfile,
+    QWebEnginePage,
+)
+from PyQt5.QtWidgets import QApplication, QWidget, QSizePolicy, QVBoxLayout
 
 from openconnect_sso import config
 
@@ -145,6 +150,11 @@ class WebBrowser(QWebEngineView):
         cookie_store.cookieAdded.connect(self._on_cookie_added)
         self.page().loadFinished.connect(self._on_load_finished)
 
+    def createWindow(self, type):
+        if type == QWebEnginePage.WebDialog:
+            self._popupWindow = WebPopupWindow(self.page().profile())
+            return self._popupWindow.view()
+
     def authenticate_at(self, url, credentials):
         script_source = pkg_resources.resource_string(__name__, "user.js").decode()
         script = QWebEngineScript()
@@ -187,6 +197,37 @@ autoFill();
         self._on_update(Url(url))
 
 
+class WebPopupWindow(QWidget):
+    def __init__(self, profile):
+        super().__init__()
+        self._view = QWebEngineView(self)
+
+        super().setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
+        super().setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
+
+        layout = QVBoxLayout()
+        super().setLayout(layout)
+        layout.addWidget(self._view)
+
+        self._view.setPage(QWebEnginePage(profile, self._view))
+
+        self._view.titleChanged.connect(super().setWindowTitle)
+        self._view.page().geometryChangeRequested.connect(
+            self.handleGeometryChangeRequested
+        )
+        self._view.page().windowCloseRequested.connect(super().close)
+
+    def view(self):
+        return self._view
+
+    @pyqtSlot("const QRect")
+    def handleGeometryChangeRequested(self, newGeometry):
+        self._view.setMinimumSize(newGeometry.width(), newGeometry.height())
+        super().move(newGeometry.topLeft() - self._view.pos())
+        super().resize(0, 0)
+        super().show()
+
+
 def to_str(qval):
     return bytes(qval).decode()
 
